package com.MPushPlugin;

import static com.MPushPlugin.push.PushUiHandler.setCallback;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;


import com.facebook.react.bridge.ActivityEventListener;
import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.LifecycleEventListener;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.Callback;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.modules.core.DeviceEventManagerModule;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import m.client.library.plugin.push.PushWNHandler;
import m.client.push.library.PushManager;
import m.client.push.library.common.Logger;
import m.client.push.library.common.PushConstants;
import m.client.push.library.utils.PushUtils;

public class MPushPlugin extends ReactContextBaseJavaModule implements ActivityEventListener, LifecycleEventListener {
    private ReactApplicationContext mReactApplicationContext;
    public static ReactContext mReactContext;
    private Boolean mHasRegisteredNotificationListener = false;

    @NonNull
    @Override
    public String getName() {
        return "mPush";
    }

    // 생성자로 초기값 설정 - 항상 해주어야함
    MPushPlugin(ReactApplicationContext context) {
        super(context);
        mReactApplicationContext = context;
        mReactContext = context;

        mReactContext.addActivityEventListener(this);
        mReactContext.addLifecycleEventListener(this);
    }

    @Override
    public void onActivityResult(Activity activity, int i, int i1, @Nullable Intent intent) {

    }

    @ReactMethod
    public void passPushData(final Callback callback) {
        Log.d("passPushData", "passPushData");
        Activity activity = mReactApplicationContext.getCurrentActivity();
        if (activity == null) {
            return;
        }
        Intent intent = activity.getIntent();
        Bundle extras;
        if (intent == null) {
            return;
        }
        extras = intent.getExtras();
        if (extras == null) {
            return;
        }

        if (mReactContext == null) {
            passPushDataToJavaScript(extras);
            return;
        }
        Bundle data = extras.getBundle("pushData");
        if (data == null) {
            return;
        }
        JSONObject pushJsonObject = new JSONObject();
        Set<String> keys = data.keySet();
        for (String key : keys) {
            try {
                pushJsonObject.put(key, data.getString(key));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
        sendEvent(mReactContext,"notification",pushJsonObject.toString());
        callback.invoke(pushJsonObject.toString());
    }


    @Override
    public void onNewIntent(Intent intent) {
        Log.e("newIntent", "NewIntet");
        String extraStr = intent.getStringExtra("moveFragment");
        if (extraStr != null) {
            if (extraStr.equals("fragment2")) {
                String notification = intent.getStringExtra("JSON");
                final String pushType = intent.getStringExtra("PUSH_TYPE");
                final String pushStatus = intent.getStringExtra("PUSH_STATUS");
                final String originData = intent.getStringExtra("ENCRYPT");
                Logger.e("================ encrypt ================");
                Logger.e(originData);
                Logger.e("================ !encrypt ================");
                JSONObject payload = null;
                try {
                    payload = new JSONObject(notification);
                    payload.put("ns", pushType);
                    notification = payload.toString();
                } catch (JSONException e) {
                    e.printStackTrace();
                }
                final String status = (pushStatus == null) ? PushConstants.APP_STATUS_START : pushStatus;
                final String payloadVal = notification;
                JSONObject tempJSON = new JSONObject();

                try {
                    tempJSON.put("payload", payloadVal);
                    tempJSON.put("type", pushType);
                    tempJSON.put("status", status);
                    tempJSON.put("encrypt", originData);

                    Log.e("MainActivity_", "Push message!!" + tempJSON.toString());
                    sendEvent(mReactContext,"notification",tempJSON.toString());
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }
    private void sendEvent(ReactContext reactContext, String eventName, String params) {
        reactContext.getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class).emit(eventName, params);
    }

    @ReactMethod
    public void addListener(String eventName) {
        Log.e("eventName::",eventName);
        // Set up any upstream listeners or background tasks as necessary
        if (eventName.equals("notification")) {
            mHasRegisteredNotificationListener = true;
        }
    }
    @ReactMethod
    public void removeListeners(Integer count) {
        // Remove upstream listeners, stop unnecessary background tasks
    }

    private void passPushDataToJavaScript(Bundle extras) {
        if (extras == null) {
            return;
        }
        Bundle data = extras.getBundle("pushData");
        if (data == null) {
            return;
        }

        // null이면 딜레이를 주면서 재귀로 실행하고 메소드 탈출
        if (mReactContext == null || !mHasRegisteredNotificationListener) {
            new Handler().postDelayed(new Runnable() {
                @Override
                public void run() {
                    Log.d("reactContext", "is not null");
                    passPushDataToJavaScript(extras);
                }
            }, 200);
            return;
        }

        JSONObject pushJsonObject = new JSONObject();
        Set<String> keys = data.keySet();
        for (String key : keys) {
            try {
                pushJsonObject.put(key, data.getString(key));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
        sendEvent(mReactContext,"notification",pushJsonObject.toString());
    }

    @ReactMethod
    public void registerServiceAndUser(ReadableMap params, Callback callback) {
        // 생성자로 설정한, context 가져오기
        ReactApplicationContext context = getReactApplicationContext();
        Log.e("test::", "registerServiceAndUser(JSONArray params)");
        final JSONObject params1 = new JSONObject();
        try {
            params1.put(PushConstants.KEY_CUID, params.getString("cuid"));
            params1.put(PushConstants.KEY_CNAME, params.getString("name"));
        } catch (JSONException e) {
            e.printStackTrace();
        }
        if (PushUtils.checkNetwork(context)) {
            setCallback(callback);
            new Handler().post(() -> {
                try {
                    PushManager.getInstance().registerServiceAndUser(context, params1);
                } catch (Exception e) {
                    Log.e("Exception:",e.getMessage());
                }
            });
        } else {
            callback.invoke("network is not connected.");
        }
    }
    @ReactMethod
    public void registerUser(ReadableMap params,Callback callback) throws JSONException {
        // 생성자로 설정한, context 가져오기
        ReactApplicationContext context = getReactApplicationContext();
        JSONObject returnValue = new JSONObject();
        returnValue.put("status", "SUCCESS");
        returnValue.put("message", "RegisterServiceAndUser를 사용하시기 바랍니다.");
        callback.invoke(returnValue.toString());
    }
    @ReactMethod
    public void registerService(ReadableMap params,Callback callback) throws JSONException {
        JSONObject returnValue = new JSONObject();
        returnValue.put("status", "SUCCESS");
        returnValue.put("message", "RegisterServiceAndUser를 사용하시기 바랍니다.");
        callback.invoke(returnValue.toString());
    }
    @ReactMethod
    public void unregisterService(Callback callback) {
        ReactApplicationContext context = getReactApplicationContext();
        if (PushUtils.checkNetwork(context)) {
            setCallback(callback);
            new Handler().post(() -> {
                // TODO Auto-generated method stub
                PushManager.getInstance().unregisterPushService(context.getApplicationContext());
            });
        } else {
            callback.invoke("network is not connected.");
        }
    }

    @ReactMethod
    public void unregisterUser(ReadableMap params, Callback callback) {
        final JSONObject params1 = new JSONObject();
        try {
            params1.put(PushConstants.KEY_CUID, params.getString("cuid"));
            params1.put(PushConstants.KEY_CNAME, params.getString("name"));
        } catch (JSONException e) {
            e.printStackTrace();
        }
        ReactApplicationContext context = getReactApplicationContext();
        if (PushUtils.checkNetwork(context)) {
            setCallback(callback);
            new Handler().post(() -> {
                // TODO Auto-generated method stub
                PushManager.getInstance().unregisterPushUser(context.getApplicationContext(), params1);
            });
        } else {
            callback.invoke("network is not connected.");
        }
    }

    @ReactMethod
    public void sendNotification(ReadableMap params, Callback callback) throws JSONException {
        Log.e("test::", "sendNotification(JSONArray params)");
        String ext = "메세지 테스트";
        // 생성자로 설정한, context 가져오기
        ReactApplicationContext context = getReactApplicationContext();
        if (PushUtils.checkNetwork(context)) {
            final String finalMessage = params.getString("message");
            final String cuid = params.getString("cuid");
            final String serviceCode = params.getString("serviceCode");
            JSONObject object1 = new JSONObject();
            object1.put("title", "일반 알림");
            object1.put("body", finalMessage);

            new Handler().post(() -> {
                PushManager.getInstance().sendPushMessage(context.getApplicationContext(), object1.toString(), ext, "", serviceCode);
            });
        } else {

        }
    }

    @ReactMethod
    public void remoteSetBadgeNumber(Callback callback) throws JSONException {
        // 생성자로 설정한, context 가져오기
        ReactApplicationContext context = getReactApplicationContext();
        if (PushUtils.checkNetwork(context)) {
            setCallback(callback);
            new Handler().post(() -> {
                PushManager.getInstance().initBadgeNo(context.getApplicationContext(), "0");
            });
        } else {
            callback.invoke("network is not connected.");
        }
    }

    @ReactMethod
    public void readNotification(ReadableMap params, Callback callback) throws JSONException {
        // 생성자로 설정한, context 가져오기
        ReactApplicationContext context = getReactApplicationContext();
        if (PushUtils.checkNetwork(context)) {
            setCallback(callback);
            new Handler().post(() -> {
                PushManager.getInstance().pushMessageReadConfirm(context.getApplicationContext(), params.getString("notification"));
            });
        } else {
            callback.invoke("network is not connected.");
        }
    }

    @ReactMethod
    public void pushMessageReceiveConfirm(ReadableMap params, Callback callback) throws JSONException {
        // 생성자로 설정한, context 가져오기
        ReactApplicationContext context = getReactApplicationContext();
        if (PushUtils.checkNetwork(context)) {
            setCallback(callback);
            new Handler().post(() -> {
                PushManager.getInstance().pushMessageReceiveConfirm(context.getApplicationContext(), params.getString("jsonString"));
            });
        } else {
            callback.invoke("network is not connected.");
        }
    }

    @ReactMethod
    public void registerGroup(ReadableMap params, Callback callback) throws JSONException {
        // 생성자로 설정한, context 가져오기
        ReactApplicationContext context = getReactApplicationContext();
        setCallback(callback);
        PushManager.getInstance().registerUserGroup(context.getApplicationContext(), params.getString("groupNum"));
    }
    @ReactMethod
    public void unRegisterGroup(ReadableMap params, Callback callback) throws JSONException {
        // 생성자로 설정한, context 가져오기
        ReactApplicationContext context = getReactApplicationContext();
        setCallback(callback);
        PushManager.getInstance().unregisterUserGroup(context.getApplicationContext(), params.getString("groupNum"));
    }

    @ReactMethod
    public void notificationCenterGetBadgeNumber(String params, Callback callback) {

        String ext = "메세지 테스트";
        // 생성자로 설정한, context 가져오기
        ReactApplicationContext context = getReactApplicationContext();
        if (PushUtils.checkNetwork(context)) {
            setCallback(callback);
            new Handler().post(() -> {
                PushManager.getInstance().initBadgeNo(context.getApplicationContext(), "10");
            });
        } else {
            callback.invoke("network is not connected.");
        }
    }

    @Override
    public Map<String, Object> getConstants() {
        final Map<String, Object> constants = new HashMap<>();
        constants.put("SHORT", Toast.LENGTH_SHORT);
        constants.put("LONG", Toast.LENGTH_LONG);
        return constants;
    }

    @Override
    public void onHostResume() {
        Activity activity = mReactApplicationContext.getCurrentActivity();
        Intent intent = activity.getIntent();

        if (intent == null) {
            return;
        }
        onNewIntent(intent);
    }

    @Override
    public void onHostPause() {

    }

    @Override
    public void onHostDestroy() {

    }
}
