package com.MPushPlugin.notification;

import m.client.push.library.PushManager;
import m.client.push.library.common.PushConstants;

import android.app.KeyguardManager;
import android.content.Context;
import android.content.Intent;
import android.util.Log;
import android.widget.Toast;

import org.json.JSONObject;

public class PushNotificationManager {
	// 마지막 메세지 체크를 위한 변수
	private static String lastestSeqNo = "";

	/**
	 * createNotification
	 * 푸시 타입에 따른 노티피케이션 생성
	 * @param context
	 * @param intent
	 * @param pushType
	 * UPNS = {"HEADER":{"ACTION":"SENDMSG"},"BODY":{"MESSAGE":"dfsfdsfsdaf","EXT":"","SEQNO":"607","PSID":"3d5c7522a0a893957d230d70d196e8dcbf3785e9","APPID":"kr.co.morpheus.mobile1","CUID":"12345","MESSAGEID":"20150212143841e6fcce8440e502b45a223a54f0","PUBLIC":"N","SENDERCODE":"admin"}}
	 * GCM = {"aps":{"badge":"1","sound":"alert.aif","alert":"fdsafasfa"},"mps":{"appid":"com.uracle.push.test","messageuniquekey":"20141204112801b6f1fd7840868afc82e51fd6a5","seqno":"387","sender":"admin",,"ext":"http://211.241.199.139:8080/msp-admin/totalInfo\\1203143401_I.html"}}
	 * http://docs.morpheus.co.kr/client/push/gcm.html#service 참고.
	 */
	public static void createNotification(final Context context, final Intent intent, final String pushType) {
		try {
			createFcmNotification(context, intent);
		} 
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * isRestrictedScreen
	 * 화면 온/오프 상태 체크 
	 * @param context
	 * @return
	 */
	public static boolean isRestrictedScreen(final Context context) {
		KeyguardManager km = (KeyguardManager) context.getSystemService(Context.KEYGUARD_SERVICE);
		return km.inKeyguardRestrictedInputMode(); 
	}
	
	/**
	 * showToast
	 * 토스트 발생 - 각 용도에 맞게 커스터마이징
	 * @param context
	 * @param intent
	 * @param pushType
	 * @throws Exception
	 */
	public static void showToast(final Context context, final Intent intent, String pushType) throws Exception {
		String jsonData = intent.getExtras().getString("JSON");
		//String psid = intent.getExtras().getString(PushConstants.KEY_PSID);
		JSONObject jsonMsg = new JSONObject(jsonData);
		
		String extData = ""; // ext 데이터
		String title = ""; // 메세지 타이틀
		String seqno = ""; // 메세지 시퀀스
		if ("UPNS".equals(pushType)) { // UPNS
			if (jsonMsg.has("EXT")) { // 존재하지 않을 수 있으므로
				extData = jsonMsg.optString("EXT");
		    }
			// 필수 항목
			title = jsonMsg.optString("MESSAGE");
		}
		else { //GCM
			JSONObject aps = jsonMsg.getJSONObject("aps");
			JSONObject mps = jsonMsg.getJSONObject("mps");
			if (aps.has("alert")) {
				title = aps.getString("alert");
		    }
		    if (mps.has("EXT")) {
		    	extData = mps.optString("EXT");
		    }
		    if (mps.has("seqno")) {
		    	seqno = mps.optString("seqno");
		    	// 중복 메세지에 대한 체크
		    	if (lastestSeqNo.equals(seqno)) {
		    		return;
		    	}
		    	lastestSeqNo = seqno;
		    }
		}
		
		Toast.makeText(context, title + ": " + extData,Toast.LENGTH_SHORT).show();
	}

	public static void createFcmNotification(final Context context, final Intent intent) throws Exception {
		String jsonData = intent.getExtras().getString("JSON");
		String encryptData = intent.getExtras().getString("message_encrypt");
		/*if (jsonData != null) {
			jsonData = jsonData.replaceAll("https", "http");
			jsonData = jsonData.replaceAll("\\\\", "/");
			jsonData = jsonData.replaceAll("//", "/");
		}*/
		try {
			Log.d("PushNotificationManager", "[PushNotificationManager] createUpnsNotification: " + jsonData);
			String psid = intent.getExtras().getString("PSID");
			JSONObject rootJsonObj = new JSONObject(jsonData);
			//PushWakeLock.acquireCpuWakeLock(context);
			FcmNotifyHelper.showNotification(context, rootJsonObj, psid, encryptData);
			PushManager.getInstance().pushMessageReceiveConfirm(context, jsonData);
			//PushWakeLock.releaseCpuLock();
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}

	public static CharSequence name = "";

}
