#import "mPush.h"
#import <MPushLibrary/MPushLibrary.h>

typedef void(^PushReceiverExtLoadHandler)(BOOL success, NSString *richData, NSError *error);

@implementation NSDictionary (MJson)

- (NSString *)jsonString {
    NSData *jsondata = [NSJSONSerialization dataWithJSONObject:self options:NSJSONWritingPrettyPrinted error:nil];
    return [[NSString alloc] initWithData:jsondata encoding:NSUTF8StringEncoding];
}

@end

@implementation mPush
RCT_EXPORT_MODULE()

- (void) initialize
{
    NSLog(@"dantexxxxxxxx mpush initialize");
    [[PushManager defaultManager] initilaizeWithDelegate:self];
    [[PushManager defaultManager] setUserNotificationDelegate];
}

// Example method
// See // https://reactnative.dev/docs/native-modules-ios

- (NSArray<NSString *> *)supportedEvents
{
    return @[@"oniOSReceiveNotification"];
}

//notificationCenterGetBadgeNumber
RCT_EXPORT_METHOD(notificationCenterGetBadgeNumber:(RCTResponseSenderBlock)result)
{
    dispatch_async(dispatch_get_main_queue(), ^{
        NSNumber *badgeNumber = [[[PushManager defaultManager] notificationCenter] badgeNumber];
        result(@[badgeNumber]);
    });
}

//notificationCenterSetBadgeNumber
RCT_EXPORT_METHOD(notificationCenterSetBadgeNumber:(NSDictionary *)param callback:(RCTResponseSenderBlock)result)
{
    int badge = [[param objectForKey:@"badge"] intValue];
    
    dispatch_async(dispatch_get_main_queue(), ^{
        [[[PushManager defaultManager] notificationCenter] setBadgeNumber:@(badge)];
        NSNumber *badgeNumber = [[[PushManager defaultManager] notificationCenter] badgeNumber];
        result(@[badgeNumber]);
    });
}

//registerService
RCT_EXPORT_METHOD(registerService:(RCTResponseSenderBlock)result)
{
    [[PushManager defaultManager] registerService:nil completionHandler:^(BOOL success) {
        if(success)
        {
            result(@[[@{@"status":@"SUCCESS"} jsonString]]);
        }
        else
        {
            result(@[[@{@"status":@"FAIL", @"error": @"register service is fail"} jsonString]]);
        }
    }];
}

//registerUser
RCT_EXPORT_METHOD(registerUser:(NSDictionary *)param callback:(RCTResponseSenderBlock)result)
{
    NSString *cuid = [param objectForKey:@"cuid"];
    NSString *name = [param objectForKey:@"name"];
    
    [[PushManager defaultManager] registerUser:nil clientUID:cuid clientName:name completionHandler:^(BOOL success) {
        if(success)
        {
            result(@[[@{@"status":@"SUCCESS"} jsonString]]);
        }
        else
        {
            result(@[[@{@"status":@"FAIL", @"error": @"register user is fail"} jsonString]]);
        }
    }];
}

//registerServiceAndUser
RCT_EXPORT_METHOD(registerServiceAndUser:(NSDictionary *)param callback:(RCTResponseSenderBlock)result)
{
    NSString *cuid = [param objectForKey:@"cuid"];
    NSString *name = [param objectForKey:@"name"];
    
    [[PushManager defaultManager] registerServiceAndUser:nil clientUID:cuid clientName:name completionHandler:^(BOOL success) {
        if(success)
        {
            result(@[[@{@"status":@"SUCCESS"} jsonString]]);
        }
        else
        {
            result(@[[@{@"status":@"FAIL", @"error": @"register serviceAndUser is fail"} jsonString]]);
        }
    }];
}

//unregisterService
RCT_EXPORT_METHOD(unregisterService:(RCTResponseSenderBlock)result)
{
    [[PushManager defaultManager] unregisterService:nil completionHandler:^(BOOL success) {
        if(success)
        {
            result(@[[@{@"status":@"SUCCESS"} jsonString]]);
        }
        else
        {
            result(@[[@{@"status":@"FAIL", @"error": @"unregister service is fail"} jsonString]]);
        }
    }];
}


//unRegisterUser
RCT_EXPORT_METHOD(unregisterUser:(RCTResponseSenderBlock)result)
{
    [[PushManager defaultManager] unregisterUser:nil completionHandler:^(BOOL success) {
        if(success)
        {
            result(@[[@{@"status":@"SUCCESS"} jsonString]]);
        }
        else
        {
            result(@[[@{@"status":@"FAIL", @"error": @"unregister user is fail"} jsonString]]);
        }
    }];
}

//remoteSetBadgeNumber
RCT_EXPORT_METHOD(remoteSetBadge:(NSDictionary *)param callback:(RCTResponseSenderBlock)result)
{
    int badge = [[param objectForKey:@"badge"] intValue];
    
    [[PushManager defaultManager] update:nil badge:@(badge) completionHandler:^(BOOL success) {
        if(success)
        {
            result(@[[@{@"status":@"SUCCESS"} jsonString]]);
        }
        else
        {
            result(@[[@{@"status":@"FAIL", @"error": @"badge update is fail"} jsonString]]);
        }
    }];
}


//readNotification
RCT_EXPORT_METHOD(readNotification:(NSDictionary *)param callback:(RCTResponseSenderBlock)result)
{
    NSDictionary *noti = [param objectForKey:@"notification"];
    [[PushManager defaultManager] read:nil notification:noti completionHandler:^(BOOL success) {
        if(success)
        {
            result(@[[@{@"status":@"SUCCESS"} jsonString]]);
        }
        else
        {
            result(@[[@{@"status":@"FAIL", @"error": @"read notification is fail"} jsonString]]);
        }
    }];
}


//sendNotification
RCT_EXPORT_METHOD(sendNotification:(NSDictionary *)param callback:(RCTResponseSenderBlock)result)
{
    NSString *cuid = [param objectForKey:@"cuid"];
    NSString *serviceCode = [param objectForKey:@"serviceCode"];
    NSString *message = [param objectForKey:@"message"];
    NSString *ext = [param objectForKey:@"ext"];
    
    [[PushManager defaultManager] send:nil clientUID:cuid message:message serviceCode:serviceCode ext:ext completionHandler:^(BOOL success) {
        if(success)
        {
            result(@[[@{@"status":@"SUCCESS"} jsonString]]);
        }
        else
        {
            result(@[[@{@"status":@"FAIL", @"error": @"sending notification is fail"} jsonString]]);
        }
    }];
}

//registerGroup
RCT_EXPORT_METHOD(registerGroup:(NSDictionary *)param callback:(RCTResponseSenderBlock)result)
{
    NSString *groupNum = [param objectForKey:@"groupNum"];
    
    [[PushManager defaultManager] registerGroup:nil groupSequenceNumber:groupNum completionHandler:^(BOOL success) {
        if(success)
        {
            result(@[[@{@"status":@"SUCCESS"} jsonString]]);
        }
        else
        {
            result(@[[@{@"status":@"FAIL", @"error": @"registerGroup is fail"} jsonString]]);
        }
    }];
}

//unregisterGroup
RCT_EXPORT_METHOD(unregisterGroup:(NSDictionary *)param callback:(RCTResponseSenderBlock)result)
{
    NSString *groupNum = [param objectForKey:@"groupNum"];
    
    [[PushManager defaultManager] unregisterGroup:nil groupSequenceNumber:groupNum completionHandler:^(BOOL success) {
        if(success)
        {
            result(@[[@{@"status":@"SUCCESS"} jsonString]]);
        }
        else
        {
            result(@[[@{@"status":@"FAIL", @"error": @"unregisterGroup is fail"} jsonString]]);
        }
    }];
}

//setEnabled
RCT_EXPORT_BLOCKING_SYNCHRONOUS_METHOD(setEnabled)
{
    [[PushManager defaultManager] setEnabled:YES];
    return @"SUCCESS";
}

//PushManagerDelegate
- (void)manager:(PushManager *)manager userNotificationCenter:(UNUserNotificationCenter *)center willPresentNotification:(UNNotification *)notification withCompletionHandler:(void (^)(UNNotificationPresentationOptions))completionHandler
{
    completionHandler(UNNotificationPresentationOptionAlert | UNNotificationPresentationOptionBadge | UNNotificationPresentationOptionSound);
}

- (void)manager:(PushManager *)manager didReceiveUserNotification:(NSDictionary *)userInfo status:(NSString *)status messageUID:(NSString *)messageUID {
    NSLog( @"PushReceiver - didReceiveUserNotification: %@ status: %@ messageUID:%@", userInfo, status, messageUID );
    
    NSString *extHTML = [[userInfo objectForKey:@"mps"] objectForKey:@"ext"];

    NSLog( @"PushReceiver - extHTML :%@", extHTML );
    
    if ( extHTML != nil && ([extHTML hasSuffix:@"_msp.html"] || [extHTML hasSuffix:@"_ext.html"]) ) {
        [self loadExtData:extHTML handler:^(BOOL success, NSString *richData, NSError *error) {
            NSLog( @"PushReceiver - richData : %@", richData );
        
            NSMutableDictionary *notification = [NSMutableDictionary dictionaryWithDictionary:userInfo];
            NSMutableDictionary *mspData = [NSMutableDictionary dictionaryWithDictionary:[notification objectForKey:@"mps"]];
            [mspData setObject:richData forKey:@"ext"];
            [notification setObject:mspData forKey:@"mps"];
            
            [self onReceiveNotification:[NSDictionary dictionaryWithDictionary:notification] status:status messageUID:messageUID];
        }];
    }
    else {
        [self onReceiveNotification:userInfo status:status messageUID:messageUID];
    }
}

- (void)loadExtData:(NSString *)extHTML handler:(PushReceiverExtLoadHandler)handler {

    NSURL *url = [NSURL URLWithString:extHTML];
    
    if (!url) {
        handler(NO, extHTML, nil);
        return;
    }

    [NSURLConnection sendAsynchronousRequest:[NSURLRequest requestWithURL:[NSURL URLWithString:extHTML]]
                            queue:[NSOperationQueue mainQueue]
                            completionHandler:^(NSURLResponse *response, NSData *data, NSError *connectionError) {
        
        if ( connectionError != nil ) {
            dispatch_async(dispatch_get_main_queue(), ^{
                handler(NO, extHTML, connectionError);
            });
            return;
        }
        
        NSHTTPURLResponse* httpResponse = (NSHTTPURLResponse *)response;
            
        if ( httpResponse.statusCode != 200 ) {
            dispatch_async(dispatch_get_main_queue(), ^{
                handler(NO, extHTML, nil);
            });
            return;
        }
        
        NSString *result = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
        NSString *richData = [NSString stringWithString:result];
        
        richData = [richData stringByRemovingPercentEncoding];
        
        #if ! __has_feature(objc_arc)
        [result release];
        #endif
        
        dispatch_async(dispatch_get_main_queue(), ^{
            handler(YES, richData, nil);
        });
    }];
}

- (void)onReceiveNotification:(NSDictionary *)notification status:(NSString *)status messageUID:(NSString *)messageUID {
    NSDictionary *noti = @{@"status":status, @"payload":notification, @"type":@"APNS", @"messageUID": messageUID};
    [self sendEventWithName:@"oniOSReceiveNotification" body:noti];
}


@end
